
#include <STM32.h>
#include "constants.h"

#include <IO_Digital.h>
#include <RTC.h>
#include <UART.h>

// Здесь мы резервируем место под новую таблицу векторов прерывания
uint32_t ramVectorTable[constants::isrVectorTableSize] __attribute__(( aligned (constants::isrVectorTableAlingment) ));
extern uint32_t g_pfnVectors[constants::isrVectorTableSize];

char     SendBuff[26];

//IO_Digital  Led_01(GPIOA, GPIO_PIN_1, Output, No_Pull);     // Для F407VET
IO_Digital  Led_01(GPIOC, GPIO_PIN_0, Output, No_Pull);     // Светодиод 1 для F407ZGT
IO_Digital  Led_02(GPIOD, GPIO_PIN_3, Output, No_Pull);     // Светодиод 2 для F407ZGT
uart        myUART(USART1, false);                          // USART1
RTC_main    myRTC(RTC);                                     // Часы реального времени

// ------------------ Объявляем "Левые" обработчики прерывания -----------------//
void setIRQ(void);

int main(void)
{
  SystemClock_Config(Quartz_8);
  /*----------------------------------------------------------------------------
   * Перекидывание векторов прерывания в ОЗУ
   * Сюда пальцы не суём
   *--------------------------------------------------------------------------*/
  for (uint16_t i = 0; i < constants::isrVectorTableSize; i++)        // Переносим вектора прерывания из FLASH
    ramVectorTable[i] = g_pfnVectors[i];                              // в ОЗУ

  __disable_irq();                                                    // Запрещаем прерывания
    SCB->VTOR = (uint32_t)&ramVectorTable;                            // Устанавливаем смещение для новой таблицы векторов прерываний
  __DSB();                                                            // Синхронизация
  __enable_irq();                                                     // Разрешием прерывания

  PeripheralUnit::setIsrVectorTable(ramVectorTable);                  // Инициализируем класс векторов
  //----------------------------------------------------------------------------
  Led_01.digitalWrite(High);                                          // Включаем первый светодидод
  myUART.init(256000);                                                // Инициализируем UART
//  myRTC.Reset();                                                      // Сброс RTC, используется один раз или при ошибках инициализации
  myRTC.Init(RTC_LSE);                                                // Инициализация RTC от кварцевого резонатора
  myRTC.setIT_WUCK(setIRQ, 1);                                        // Инициализация внешнего обработчика прерываний с таймингом в 1 секунду
//  myRTC.setTime(15, 55, 0);
//  myRTC.setDate(10, 6, 2022, 5);

  while (1)
  {
    bool interruptOccured = myRTC.getInterruptFlag();                 // Считываем флаг
    if (interruptOccured == true)                                     // Если флаг от RTC поднят,
    {
      Led_01.digitalTogglePin();                                      // Переключаем 1й светодиод,
      myRTC.clearInterruptFlag();                                     // и сбрасываем флаг от RTC.
    }
  }
}

/*
 * Это внешний обработчик прерывания.
 * Переключает 2й светодиод и выводит текущую дату и время в UART
 */
void setIRQ(void)
{
  Led_02.digitalTogglePin();
  myRTC.getDates();
  sprintf(SendBuff, "Date : %02d/%02d/%04d - %d", myRTC.Dates.Date, myRTC.Dates.Month, myRTC.Dates.Year, myRTC.Dates.Week);
  myUART.print(SendBuff);
  myRTC.getTime();
  sprintf(SendBuff, "  Time : %02d:%02d:%02d", myRTC.Time.Hour, myRTC.Time.Minute, myRTC.Time.Sec);
  myUART.println(SendBuff);
}
/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */
  __disable_irq();
  while (1)
  {
  }
  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
